// Filesystem restriction configs (internal structures built from permission rules)
export {};//# sourceMappingURL=https://github.com/VSCodium/sourcemaps/releases/download/stable-4c0b0c6cc561d2d3636d1ec250935431876ce4dc/node_modules-@vscode-sandbox-runtime-dist-sandbox-sandbox-schemas.js.map